@@ -31,7 +31,6 @@
 #include <gdk/gdkx.h>
 #include <math.h>
 
-#include "capplet-util.h"
 #include "gconf-property-editor.h"
 #include "gnome-mouse-accessibility.h"
 #include "capplet-stock-icons.h"
@@ -48,6 +47,8 @@
 #include <X11/Xcursor/Xcursor.h>
 #endif
 
+#define WID(x) (GtkWidget*) gtk_builder_get_object (dialog, x)
+
 enum
 {
 	DOUBLE_CLICK_TEST_OFF,
